@@ -357,7 +357,7 @@ bool Parser::parseSkins()
         const boost::property_tree::ptree& pSkinTree = itor->second;
         const boost::property_tree::ptree& pShapeTree =
             pSkinTree.get_child("bindShapeMatrix");
-        float buffer[16];
+        float buffer[16] = {0.0f};
         unsigned int i = 0;
         boost::property_tree::ptree::const_iterator it;
         boost::property_tree::ptree::const_iterator itEnd = pShapeTree.end();
@@ -488,7 +488,10 @@ int Parser::parseMaterials(const std::vector<glTFFile>& inputFiles)
             pMaterial, techniqueId, inputFiles);
 
         if( nRet != LIBGLTF_SUCCESS )
+        {
+            delete pMaterial;
             return nRet;
+        }
 
         pMaterial->setTechniqueId(techniqueId);
         pScene->insertMaterialMap(it->first, pMaterial);
@@ -502,6 +505,7 @@ int Parser::parseMaterialProper(const boost::property_tree::ptree& pTree,
                                  const std::string& techniqueId,
                                  const std::vector<glTFFile>& inputFiles)
 {
+    int nRet = LIBGLTF_SUCCESS;
     std::string properName;
     boost::property_tree::ptree::const_iterator it = pTree.begin();
     boost::property_tree::ptree::const_iterator itEnd =  pTree.end();
@@ -534,9 +538,7 @@ int Parser::parseMaterialProper(const boost::property_tree::ptree& pTree,
                         imagePath.c_str(), '*')));
                     pMaterialProper->setDataLength(0);
                     pMaterialProper->setDataType(DataType_SAMPLER_2D);
-                    int nRet = pScene->loadTexture(pMaterialProper->getImagePath(), inputFiles);
-                    if( nRet != LIBGLTF_SUCCESS )
-                        return nRet;
+                    nRet = pScene->loadTexture(pMaterialProper->getImagePath(), inputFiles);
                 }
                 break;
             case DataType_FLOAT:
@@ -575,7 +577,7 @@ int Parser::parseMaterialProper(const boost::property_tree::ptree& pTree,
         pMaterial->pushMaterialProper(pMaterialProper);
     }
 
-    return LIBGLTF_SUCCESS;
+    return nRet;
 }
 
 bool Parser::parseCameras()
@@ -772,41 +774,6 @@ bool Parser::parseAttributes()
                         + pBufferView.get<unsigned int>("byteOffset")
                         + pAttrTree.get<unsigned int>("byteOffset");
         pAttribute->setAttributeData(pbuffer, count * byteStride);
-
-        if (DataType_FLOAT_VEC3 == type)
-        {
-           boost::property_tree::ptree::const_assoc_iterator itAttr;
-           itAttr = pAttrTree.find("max");
-           if (itAttr != pAttrTree.not_found())
-           {
-              float max[3] = {0.f, 0.f, 0.f};
-              const boost::property_tree::ptree& pMaxTree = itAttr->second;
-              boost::property_tree::ptree::const_iterator iter;
-              iter = pMaxTree.begin();
-              boost::property_tree::ptree::const_iterator iterEnd;
-              iterEnd = pMaxTree.end();
-              for (unsigned int i = 0; (iter != iterEnd) && (i<3); ++iter, ++i)
-              {
-                  max[i] = (iter->second).get_value<float>();
-              }
-              pScene->setVertexMax(max[0], max[1], max[2]);
-          }
-          itAttr = pAttrTree.find("min");
-          if (itAttr != pAttrTree.not_found())
-          {
-              float min[3] = {0.f, 0.f, 0.f};
-              const boost::property_tree::ptree& pMinTree = itAttr->second;
-              boost::property_tree::ptree::const_iterator iter;
-              iter = pMinTree.begin();
-              boost::property_tree::ptree::const_iterator iterEnd;
-              iterEnd = pMinTree.end();
-              for (unsigned int i = 0; (iter != iterEnd) && (i<3); ++iter, ++i)
-              {
-                  min[i++] = (iter->second).get_value<float>();
-              }
-              pScene->setVertexMin(min[0], min[1], min[2]);
-          }
-        }
         pScene->insertAttributeMap(it->first, pAttribute);
     }
     pAttrsTree.clear();
@@ -835,7 +802,10 @@ int Parser::parseTechniques(const std::vector<glTFFile>& inputFiles)
         int status = parseTechniqueProgram(pInstanceTree,
             pParameterTree, pTechnique, inputFiles);
         if (status != LIBGLTF_SUCCESS)
+        {
+            delete pTechnique;
             return status;
+        }
         std::string statePath = "passes*" + passId + "*states";
         boost::property_tree::ptree pStateTree =
             pTechniqueTree.get_child(
