@@ -343,29 +343,25 @@ static void integer_to_text(const dsc* from, dsc* to, Callbacks* cb)
 		u = -n;
 	}
 
-    UCHAR temp[32];
-    UCHAR* p = temp;
+    string temp;
 
 	do {
-		*p++ = (UCHAR) (u % 10) + '0';
+		temp += (u % 10) + '0';
 		u /= 10;
 	} while (u);
 
-	SSHORT l = p - temp;
+	SSHORT l = (SSHORT) temp.length();
 
 	// if scale < 0, we need at least abs(scale)+1 digits, so add
 	// any leading zeroes required.
 	while (l + scale <= 0)
 	{
-		*p++ = '0';
+		temp += '0';
 		l++;
 	}
 	// postassertion: l+scale > 0
 	fb_assert(l + scale > 0);
 
-	// CVC: also, we'll check for buffer overflow directly.
-	fb_assert(temp + sizeof(temp) >= p);
-
 	// Compute the total length of the field formatted.  Make sure it
 	// fits.  Keep in mind that routine handles both string and varying
 	// string fields.
@@ -379,6 +375,7 @@ static void integer_to_text(const dsc* from, dsc* to, Callbacks* cb)
 	    CVT_conversion_error(from, cb->err);
 	}
 
+	const UCHAR* p = (UCHAR*) temp.c_str() + temp.length();
 	UCHAR* q = (to->dsc_dtype == dtype_varying) ? to->dsc_address + sizeof(USHORT) : to->dsc_address;
 	const UCHAR* start = q;
 
@@ -1286,6 +1283,21 @@ void CVT_move_common(const dsc* from, dsc* to, Callbacks* cb)
 		return;
 	}
 
+	// Special optimization case: RDB$DB_KEY is binary compatible with CHAR(8) OCTETS
+
+	if ((from->dsc_dtype == dtype_text &&
+		to->dsc_dtype == dtype_dbkey &&
+		from->dsc_ttype() == ttype_binary &&
+		from->dsc_length == to->dsc_length) ||
+		(to->dsc_dtype == dtype_text &&
+		from->dsc_dtype == dtype_dbkey &&
+		to->dsc_ttype() == ttype_binary &&
+		from->dsc_length == to->dsc_length))
+	{
+		memcpy(p, q, length);
+		return;
+	}
+
 	// Do data type by data type conversions.  Not all are supported,
 	// and some will drop out for additional handling.
 
@@ -1530,7 +1542,7 @@ void CVT_move_common(const dsc* from, dsc* to, Callbacks* cb)
 				// multibyte character sets which use a zero
 				// byte to represent end-of-string
 
-				fb_assert(to->dsc_length > 0)
+				fb_assert(to->dsc_length > 0);
 				length = MIN(length, ULONG(to->dsc_length - 1));
 				cb->validateData(toCharset, length, q);
 				toLength = length;
@@ -1724,9 +1736,9 @@ void CVT_conversion_error(const dsc* desc, ErrorFunction err)
 		try
 	    {
 			const char* p;
-			VaryStr<41> s;
+			VaryStr<128> s;
 			const USHORT length =
-				CVT_make_string(desc, ttype_ascii, &p, &s, sizeof(s) - 1, localError);
+				CVT_make_string(desc, ttype_ascii, &p, &s, sizeof(s), localError);
 			message.assign(p, length);
 		}
 		/*
@@ -2225,7 +2237,9 @@ USHORT CVT_get_string_ptr_common(const dsc* desc, USHORT* ttype, UCHAR** address
 	fb_assert(cb != NULL);
 	fb_assert((temp != NULL && length > 0) ||
 			desc->dsc_dtype == dtype_text ||
-			desc->dsc_dtype == dtype_cstring || desc->dsc_dtype == dtype_varying);
+			desc->dsc_dtype == dtype_cstring ||
+			desc->dsc_dtype == dtype_varying ||
+			desc->dsc_dtype == dtype_dbkey);
 
 	// If the value is already a string (fixed or varying), just return
 	// the address and length.
