@@ -50,13 +50,21 @@ static int printUsage()
 	printf("Options:\n");
 	printf("\t-h:          Shows this help message\n");
 	printf("\t-dc:         Sets the decimal commas to character c: default .\n");
+	printf("\t-e \"encoding\":    Define the file encoding where encoding can be\n");
+	printf("\t\t CP037, CP424, CP437, CP737, CP500, CP775, CP850, CP852, CP855, CP856, CP857,\n");
+	printf("\t\t CP860, CP861, CP862, CP863, CP864, CP865, CP866, CP869, CP874, CP875, CP932,\n");
+	printf("\t\t CP950, CP1006, CP1026, CP1250, CP1251, CP1252, CP1253, CP1254, CP1255, CP1256,\n");
+	printf("\t\t CP1257, CP1258, MacArabic, MacCeltic, MacCEurope, MacCroation, MacCyrillic,\n");
+	printf("\t\t MacDevanage, MacFarsi, MacGaelic, MacGreek, MacGujarati, MacGurmukhi, MacHebrew,\n");
+	printf("\t\t MacIceland, MacInuit, MacRoman, MacRomanian, MacThai, MacTurkish.\n");
 	printf("\t-fc:         Sets the field separator to character c: default ,\n");
 	printf("\t-tc:         Sets the text separator to character c: default \"\n");
+	printf("\t-o file.csv: Defines the ouput file\n");
+	printf("\t-p password: Password to open the file\n");
+	printf("\t-v:          Output wks2csv version\n");
 	printf("\t-F:          Sets to output the formula which exists in the file\n");
 	printf("\t-Dformat:    Sets the date format: default \"%%m/%%d/%%y\"\n");
 	printf("\t-Tformat:    Sets the time format: default \"%%H:%%M:%%S\"\n");
-	printf("\t-o file.csv: Defines the ouput file\n");
-	printf("\t-v:          Output wks2csv version\n");
 	printf("\n");
 	printf("Example:\n");
 	printf("\twks2cvs -d, -D\"%%d/%%m/%%y\" file : Converts a file using french locale\n");
@@ -76,12 +84,14 @@ int main(int argc, char *argv[])
 {
 	bool printHelp=false;
 	bool generateFormula=false;
+	char const *encoding="";
+	char const *password=0;
 	char const *output = 0;
 	int ch;
 	char decSeparator='.', fieldSeparator=',', textSeparator='"';
 	std::string dateFormat("%m/%d/%y"), timeFormat("%H:%M:%S");
 
-	while ((ch = getopt(argc, argv, "hvo:d:f:t:D:FT:")) != -1)
+	while ((ch = getopt(argc, argv, "e:hvo:d:f:p:t:D:FT:")) != -1)
 	{
 		switch (ch)
 		{
@@ -97,6 +107,9 @@ int main(int argc, char *argv[])
 		case 'd':
 			decSeparator=optarg[0];
 			break;
+		case 'e':
+			encoding=optarg;
+			break;
 		case 'f':
 			fieldSeparator=optarg[0];
 			break;
@@ -106,6 +119,9 @@ int main(int argc, char *argv[])
 		case 'o':
 			output=optarg;
 			break;
+		case 'p':
+			password=optarg;
+			break;
 		case 'v':
 			printVersion();
 			return 0;
@@ -123,9 +139,11 @@ int main(int argc, char *argv[])
 	char const *file=argv[optind];
 	librevenge::RVNGFileStream input(file);
 
+	WPSCreator creator;
 	WPSKind kind;
-	WPSConfidence confidence = WPSDocument::isFileFormatSupported(&input,kind);
-	if (confidence == WPS_CONFIDENCE_NONE)
+	bool needCharEncoding;
+	WPSConfidence confidence = WPSDocument::isFileFormatSupported(&input,kind,creator,needCharEncoding);
+	if (confidence == WPS_CONFIDENCE_NONE || (kind != WPS_SPREADSHEET && kind != WPS_DATABASE))
 	{
 		printf("ERROR: Unsupported file format!\n");
 		return 1;
@@ -133,25 +151,22 @@ int main(int argc, char *argv[])
 
 	WPSResult error=WPS_OK;
 	librevenge::RVNGStringVector vec;
-	if (kind != WPS_SPREADSHEET && kind != WPS_DATABASE)
-	{
-		printf("ERROR: Unsupported file format!\n");
-		return 1;
-	}
 
 	try
 	{
 		librevenge::RVNGCSVSpreadsheetGenerator listenerImpl(vec, generateFormula);
 		listenerImpl.setSeparators(fieldSeparator, textSeparator, decSeparator);
 		listenerImpl.setDTFormats(dateFormat.c_str(),timeFormat.c_str());
-		error= WPSDocument::parse(&input, &listenerImpl);
+		error= WPSDocument::parse(&input, &listenerImpl, password, encoding);
 	}
 	catch (...)
 	{
 		error=WPS_PARSE_ERROR;
 	}
 
-	if (error == WPS_FILE_ACCESS_ERROR)
+	if (error == WPS_ENCRYPTION_ERROR)
+		fprintf(stderr, "ERROR: Encrypted file, bad Password!\n");
+	else if (error == WPS_FILE_ACCESS_ERROR)
 		fprintf(stderr, "ERROR: File Exception!\n");
 	else if (error == WPS_PARSE_ERROR)
 		fprintf(stderr, "ERROR: Parse Exception!\n");
