@@ -338,7 +338,7 @@ struct Zone {
   struct Information;
 
   //! constructor
-  Zone(int zId) : m_id(zId), m_infoList(), m_fontList(),m_rulerList(), m_idFontMap(), m_posFontMap(), m_posRulerMap(), m_actZone(0), m_parsed(false)
+  explicit Zone(int zId) : m_id(zId), m_infoList(), m_fontList(),m_rulerList(), m_idFontMap(), m_posFontMap(), m_posRulerMap(), m_actZone(0), m_parsed(false)
   {
   }
 
@@ -437,7 +437,7 @@ struct Table {
   struct Cell;
   struct Row;
   //! constructor
-  Table(Zone const &zone) : m_zone(zone), m_rowsList()
+  explicit Table(Zone const &zone) : m_zone(zone), m_rowsList()
   {
   }
   //! returns the next char position after the table
@@ -1411,7 +1411,7 @@ bool MarinerWrtText::readFonts(MarinerWrtEntry const &entry, int zoneId)
             font.m_font.setStrikeOutStyle(MWAWFont::Line::Simple);
             break;
           case 54:
-            fFlags |= MWAWFont::allCapsBit;
+            fFlags |= MWAWFont::uppercaseBit;
             break;
           case 55:
             fFlags |= MWAWFont::lowercaseBit;
