@@ -161,6 +161,9 @@ public:
   //! constructor
   FontName() : m_name(""), m_id(-1), m_unknown(0) { }
 
+  //! operator<<
+  friend std::ostream &operator<<(std::ostream &o, FontName const &ft);
+
   //! the font name
   std::string m_name;
   //! the font id
@@ -168,7 +171,7 @@ public:
   //! unknown
   int m_unknown;
 };
-//! operator<< for a font name
+
 std::ostream &operator<<(std::ostream &o, FontName const &ft)
 {
   o << "Font(name=" << ft.m_name << ", id=" << ft.m_id;
@@ -184,7 +187,7 @@ struct Font {
   //! constructor: default value Geneva:12
   Font(int fId=3, int size=12) :m_font(fId,float(size)), m_fieldType(None), m_error("") {}
   //! copy constructor
-  Font(MWAWFont const &ft) :m_font(ft), m_fieldType(None), m_error("") {}
+  explicit Font(MWAWFont const &ft) :m_font(ft), m_fieldType(None), m_error("") {}
   //! operator <<
   friend std::ostream &operator<<(std::ostream &o, Font const &ft);
   //! the font
@@ -336,7 +339,7 @@ struct Object {
   //! the local id
   int m_id;
   //! the dimension
-  Vec2i m_dim;
+  MWAWVec2i m_dim;
   //! the file id
   long m_fileId;
   //! a string used to store the parsing errors
@@ -1041,7 +1044,13 @@ bool MsWks4Text::readFontNames(MWAWInputStreamPtr input, MWAWEntry const &entry)
   if (int(len)+10 != entry.length())
     f << ", ###size=" << std::hex << len+10 << std::dec ;
   for (int i = 0; i < 3; i++) f << ", " << input->readLong(2); // -1/4/0
-
+  if (debPos+10+2*long(n_fonts)>endPos) {
+    MWAW_DEBUG_MSG(("MsWks4Text::readFontNames: the number of font seems bad\n"));
+    f << "###";
+    ascFile.addPos(debPos);
+    ascFile.addNote(f.str().c_str());
+    return false;
+  }
   f << ", defPos=[" << std::hex; // FIXME: used it
   for (int i = 0; i < int(n_fonts); i++)
     f << debPos+10+input->readLong(2) << ", ";
@@ -1079,7 +1088,7 @@ bool MsWks4Text::readFontNames(MWAWInputStreamPtr input, MWAWEntry const &entry)
 
   if (m_state->m_fontNames.size() != n_fonts) {
     MWAW_DEBUG_MSG(("MsWks4Text::readFontNames: warning: expected %i fonts but only found %i\n",
-                    n_fonts, int(m_state->m_fontNames.size())));
+                    (int) n_fonts, (int)m_state->m_fontNames.size()));
     return false;
   }
   return true;
@@ -1367,7 +1376,7 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
       parag.m_listLevelIndex = 1;
       break;
     }
-    case 0x23: { // alignement -2 (up space), -3 (large?)
+    case 0x23: { // alignment -2 (up space), -3 (large?)
       if (pos + 2 > endPos) {
         ok = false;
         break;
@@ -1458,7 +1467,7 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
       input->seek(pos+42, librevenge::RVNG_SEEK_SET);
       break;
     }
-    case 0x28: { // alignement: exact or not
+    case 0x28: { // alignment: exact or not
       if (pos + 1 > endPos) {
         ok = false;
         break;
@@ -1816,7 +1825,7 @@ bool MsWks4Text::readFDP(MWAWInputStreamPtr &input, MWAWEntry const &entry,
 
   if (length < headerSize) {
     MWAW_DEBUG_MSG(("MsWks4Text::readFDP: warning: FDP offset=0x%X, length=0x%lx\n",
-                    page_offset, length));
+                    page_offset, (long unsigned int) length));
     return false;
   }
 
@@ -1829,7 +1838,7 @@ bool MsWks4Text::readFDP(MWAWInputStreamPtr &input, MWAWEntry const &entry,
   f << ", unk=" << input->readLong(2);
 
   if (headerSize+(4+deplSize)*cfod > length) {
-    MWAW_DEBUG_MSG(("MsWks4Text::readFDP: error: cfod = %i (0x%X)\n", cfod, cfod));
+    MWAW_DEBUG_MSG(("MsWks4Text::readFDP: error: cfod = %i (0x%X)\n", cfod, (unsigned int) cfod));
     return false;
   }
 
@@ -1885,7 +1894,7 @@ bool MsWks4Text::readFDP(MWAWInputStreamPtr &input, MWAWEntry const &entry,
     if ((depl < headerSize+(4+deplSize)*cfod && depl > 0) ||
         long(page_offset)+depl  > endPage) {
       MWAW_DEBUG_MSG(("MsWks4Text::readFDP: error: pos of bfprop is bad "
-                      "%i (0x%X)\n", depl, depl));
+                      "%i (0x%X)\n", depl, (unsigned int) depl));
       return false;
     }
 
@@ -1917,7 +1926,7 @@ bool MsWks4Text::readFDP(MWAWInputStreamPtr &input, MWAWEntry const &entry,
     int szProp = (int) input->readULong(1);
     szProp++;
     if (szProp == 0) {
-      MWAW_DEBUG_MSG(("MsWks4Text::readFDP: error: 0 == szProp at file offset 0x%lx\n", (input->tell())-1));
+      MWAW_DEBUG_MSG(("MsWks4Text::readFDP: error: 0 == szProp at file offset 0x%lx\n", (unsigned long int)(input->tell()-1)));
       return false;
     }
     long endPos = pos+szProp;
