@@ -29,51 +29,51 @@
 #include <vector>
 #include <sstream>
 
-WP3ResourceFork::WP3ResourceFork(WPXInputStream *input, WPXEncryption *encryption) :
+WP3ResourceFork::WP3ResourceFork(librevenge::RVNGInputStream *input, WPXEncryption *encryption) :
 	m_resourcesTypeMultimap(), m_resourcesIDMultimap()
 {
-	input->seek(16, WPX_SEEK_SET);
-	uint32_t dataOffset = readU32(input, encryption, true);
-	uint32_t mapOffset = readU32(input, encryption, true);
-	/* uint32_t dataLength = */
+	input->seek(16, librevenge::RVNG_SEEK_SET);
+	unsigned dataOffset = readU32(input, encryption, true);
+	unsigned mapOffset = readU32(input, encryption, true);
+	/* unsigned dataLength = */
 	readU32(input, encryption, true);
-	/* uint32_t mapLength = */
+	/* unsigned mapLength = */
 	readU32(input, encryption, true);
 
-	input->seek(40+mapOffset, WPX_SEEK_SET);
+	input->seek(40+mapOffset, librevenge::RVNG_SEEK_SET);
 
-	uint32_t typeOffset = readU16(input, encryption, true);
-	uint32_t nameListOffset = readU16(input, encryption, true);
+	unsigned typeOffset = readU16(input, encryption, true);
+	unsigned nameListOffset = readU16(input, encryption, true);
 
-	input->seek(16+typeOffset+mapOffset, WPX_SEEK_SET);
-	uint16_t resourceTypesNumber = (uint16_t)(1 + readU16(input, encryption, true));
+	input->seek(16+typeOffset+mapOffset, librevenge::RVNG_SEEK_SET);
+	unsigned short resourceTypesNumber = (unsigned short)(1 + readU16(input, encryption, true));
 
 	for (unsigned i=0; i < resourceTypesNumber; i++)
 	{
-		uint32_t resourceType = readU32(input,encryption, true);
-		uint32_t resourcesNumber = 1+readU16(input, encryption, true);
-		uint32_t referenceListOffset = 16+typeOffset+mapOffset+readU16(input, encryption, true);
+		unsigned resourceType = readU32(input,encryption, true);
+		unsigned resourcesNumber = 1+readU16(input, encryption, true);
+		unsigned referenceListOffset = 16+typeOffset+mapOffset+readU16(input, encryption, true);
 		long position = input->tell();
-		input->seek(referenceListOffset, WPX_SEEK_SET);
-		for (uint32_t j=0; j < resourcesNumber; j++)
+		input->seek(referenceListOffset, librevenge::RVNG_SEEK_SET);
+		for (unsigned j=0; j < resourcesNumber; j++)
 		{
-			uint32_t resourceReferenceID = readU16(input, encryption, true);
-			uint16_t resourceNameOffset = readU16(input, encryption, true);
-			WPXString resourceName;
+			unsigned resourceReferenceID = readU16(input, encryption, true);
+			unsigned short resourceNameOffset = readU16(input, encryption, true);
+			librevenge::RVNGString resourceName;
 			if (resourceNameOffset != 0xffff)
 			{
 				long position2 = input->tell();
-				input->seek(16+mapOffset+nameListOffset+resourceNameOffset, WPX_SEEK_SET);
+				input->seek(16+mapOffset+nameListOffset+resourceNameOffset, librevenge::RVNG_SEEK_SET);
 				resourceName = readPascalString(input, encryption);
-				input->seek(position2, WPX_SEEK_SET);
+				input->seek(position2, librevenge::RVNG_SEEK_SET);
 			}
-			uint8_t resourceAttributes = readU8(input, encryption);
-			uint32_t offsetToData = (uint32_t)((uint32_t)readU8(input, encryption) << 16);
+			unsigned char resourceAttributes = readU8(input, encryption);
+			unsigned offsetToData = (unsigned)((unsigned)readU8(input, encryption) << 16);
 			offsetToData |= readU16(input, encryption, true);
 			offsetToData += 16+dataOffset;
 			long position3 = input->tell();
-			input->seek(offsetToData, WPX_SEEK_SET);
-			uint32_t resourceDataSize = readU32(input, encryption, true);
+			input->seek(offsetToData, librevenge::RVNG_SEEK_SET);
+			unsigned resourceDataSize = readU32(input, encryption, true);
 
 			unsigned long oldEncryptionOffset = 0;
 			unsigned char oldEncryptionMaskBase = 0;
@@ -89,8 +89,8 @@ WP3ResourceFork::WP3ResourceFork(WPXInputStream *input, WPXEncryption *encryptio
 				}
 			}
 
-			WPXBinaryData resourceData;
-			for (unsigned long k = 0; k < (unsigned long)resourceDataSize && !input->atEOS(); k++)
+			librevenge::RVNGBinaryData resourceData;
+			for (unsigned long k = 0; k < (unsigned long)resourceDataSize && !input->isEnd(); k++)
 				resourceData.append((unsigned char)readU8(input, encryption));
 
 			if (encryption)
@@ -99,16 +99,16 @@ WP3ResourceFork::WP3ResourceFork(WPXInputStream *input, WPXEncryption *encryptio
 				encryption->setEncryptionMaskBase(oldEncryptionMaskBase);
 			}
 
-			input->seek(position3, WPX_SEEK_SET);
+			input->seek(position3, librevenge::RVNG_SEEK_SET);
 			WP3Resource *resource = new WP3Resource(resourceType, resourceReferenceID, resourceName, resourceAttributes, resourceData);
-			m_resourcesTypeMultimap.insert(std::multimap<uint32_t, WP3Resource *>::value_type( resourceType, resource ) );
-			m_resourcesIDMultimap.insert(std::multimap<uint32_t, WP3Resource *>::value_type( resourceReferenceID, resource ) );
+			m_resourcesTypeMultimap.insert(std::multimap<unsigned, WP3Resource *>::value_type(resourceType, resource));
+			m_resourcesIDMultimap.insert(std::multimap<unsigned, WP3Resource *>::value_type(resourceReferenceID, resource));
 			WPD_DEBUG_MSG(("WP3Resource: Type 0x%.8x, ID %i, name %s, attributes 0x%.2x\n", resourceType, resourceReferenceID, resourceName.cstr(), resourceAttributes));
-			input->seek(4, WPX_SEEK_CUR);
+			input->seek(4, librevenge::RVNG_SEEK_CUR);
 #if 0
-			WPXInputStream *tmpBinaryStream = const_cast<WPXInputStream *>(resourceData.getDataStream());
+			librevenge::RVNGInputStream *tmpBinaryStream = resourceData.getDataStream();
 			int indexNumber = 0;
-			while (!tmpBinaryStream->atEOS())
+			while (!tmpBinaryStream->isEnd())
 			{
 				if (!(indexNumber % 16))
 					printf("%.8x: ", indexNumber);
@@ -125,40 +125,40 @@ WP3ResourceFork::WP3ResourceFork(WPXInputStream *input, WPXEncryption *encryptio
 			std::ostringstream filename;
 			filename << "binarydump" << m_resourcesTypeMultimap.size() << ".bin";
 			FILE *f = fopen(filename.str().c_str(), "wb");
-			WPXBinaryData tmpResData;
+			librevenge::RVNGBinaryData tmpResData;
 			for (int tmpIndex = 0; tmpIndex < 512; tmpIndex++)
 				tmpResData.append((const unsigned char)0);
 			tmpResData.append(resourceData);
 			if (f)
 			{
-				WPXInputStream *tmpStream = const_cast<WPXInputStream *>(tmpResData.getDataStream());
-				while (!tmpStream->atEOS())
+				librevenge::RVNGInputStream *tmpStream = tmpResData.getDataStream();
+				while (!tmpStream->isEnd())
 					fprintf(f, "%c", readU8(tmpStream, 0));
 				fclose(f);
 			}
 #endif
 		}
-		input->seek(position, WPX_SEEK_SET);
+		input->seek(position, librevenge::RVNG_SEEK_SET);
 	}
 }
 
 WP3ResourceFork::~WP3ResourceFork()
 {
 	// delete the pointers from one of the multimaps.
-	for (std::multimap<uint32_t, WP3Resource *>::iterator iter = m_resourcesTypeMultimap.begin(); iter != m_resourcesTypeMultimap.end(); ++iter)
+	for (std::multimap<unsigned, WP3Resource *>::iterator iter = m_resourcesTypeMultimap.begin(); iter != m_resourcesTypeMultimap.end(); ++iter)
 		delete iter->second;
 }
 
-const WP3Resource *WP3ResourceFork::getResource(uint32_t type, uint32_t ID) const
+const WP3Resource *WP3ResourceFork::getResource(unsigned type, unsigned ID) const
 {
-	std::pair< std::multimap<uint32_t, WP3Resource *>::const_iterator, std::multimap<uint32_t, WP3Resource *>::const_iterator > tempPair
+	std::pair< std::multimap<unsigned, WP3Resource *>::const_iterator, std::multimap<unsigned, WP3Resource *>::const_iterator > tempPair
 	    = m_resourcesTypeMultimap.equal_range(type);
 
 	if (tempPair.first == m_resourcesTypeMultimap.end())
 		return NULL;
 
-	for (std::multimap<uint32_t, WP3Resource *>::const_iterator iter = tempPair.first; iter != tempPair.second; ++iter)
-		if (iter->second->getResourceReferenceID() == ID )
+	for (std::multimap<unsigned, WP3Resource *>::const_iterator iter = tempPair.first; iter != tempPair.second; ++iter)
+		if (iter->second->getResourceReferenceID() == ID)
 			return iter->second;
 
 	return NULL;
