@@ -28,13 +28,13 @@
 #define WP42HEURISTICS_H
 
 #include <libwpd/libwpd.h>
-#include <libwpd-stream/libwpd-stream.h>
+#include <librevenge-stream/librevenge-stream.h>
 
 class WP42Heuristics
 {
 public:
-	static WPDConfidence isWP42FileFormat(WPXInputStream *input, const char *password = 0);
-	static WPDPasswordMatch verifyPassword(WPXInputStream *input, const char *password);
+	static libwpd::WPDConfidence isWP42FileFormat(librevenge::RVNGInputStream *input, const char *password = 0);
+	static libwpd::WPDPasswordMatch verifyPassword(librevenge::RVNGInputStream *input, const char *password);
 };
 
 #endif /* WP42HEURISTICS_H */
