@@ -32,8 +32,8 @@
 class WP5Header : public WPXHeader
 {
 public:
-	WP5Header(WPXInputStream *input, WPXEncryption *encryption, uint32_t documentOffset, uint8_t productType,
-	          uint8_t fileType, uint8_t majorVersion, uint8_t minorVersion, uint16_t documentEncryption);
+	WP5Header(librevenge::RVNGInputStream *input, WPXEncryption *encryption, unsigned documentOffset, unsigned char productType,
+	          unsigned char fileType, unsigned char majorVersion, unsigned char minorVersion, unsigned short documentEncryption);
 	~WP5Header() {}
 };
 #endif /* _WP5HEADER_H  */
