@@ -68,7 +68,7 @@ void WPSTabStop::addTo(librevenge::RVNGPropertyListVector &propList, double deca
 
 	// position
 	double position = m_position+decalX;
-	if (position < 0.00005f && position > -0.00005f)
+	if (position < 0.00005 && position > -0.00005)
 		position = 0.0;
 	tab.insert("style:position", position);
 
@@ -117,7 +117,18 @@ std::ostream &operator<<(std::ostream &o, WPSParagraph const &pp)
 		o << "rightMarg=" << pp.m_margins[2] << ",";
 
 	if (pp.m_spacings[0] < 1.0 || pp.m_spacings[0] > 1.0)
-		o << "interLineSpacing=" << pp.m_spacings[0] << ",";
+	{
+		o << "interLineSpacing=" << pp.m_spacings[0];
+		if (pp.m_spacingsInterlineUnit==librevenge::RVNG_PERCENT)
+			o << "%";
+		else if (pp.m_spacingsInterlineUnit==librevenge::RVNG_POINT)
+			o << "pt";
+		else if (pp.m_spacingsInterlineUnit==librevenge::RVNG_INCH)
+			o << "in";
+		if (pp.m_spacingsInterlineType==WPSParagraph::AtLeast)
+			o << "[atLeast]";
+		o << ",";
+	}
 	if (pp.m_spacings[1]<0||pp.m_spacings[1]>0)
 		o << "befSpacing=" << pp.m_spacings[1] << ",";
 	if (pp.m_spacings[2]<0||pp.m_spacings[2]>0)
@@ -143,7 +154,7 @@ std::ostream &operator<<(std::ostream &o, WPSParagraph const &pp)
 		o << "just=fullAllLines, ";
 		break;
 	default:
-		assert(false); // unhandled Justification enum value
+		WPS_DEBUG_MSG(("WPSParagraph:operator<<: called with unknown justification\n"));
 		o << "just=" << static_cast<unsigned>(pp.m_justify) << ", ";
 		break;
 	}
@@ -155,8 +166,8 @@ std::ostream &operator<<(std::ostream &o, WPSParagraph const &pp)
 			o << pp.m_tabs[i] << ",";
 		o << "),";
 	}
-	if (pp.m_backgroundColor != 0xFFFFFF)
-		o << "backgroundColor=" << std::hex << pp.m_backgroundColor << std::dec << ",";
+	if (!pp.m_backgroundColor.isWhite())
+		o << "backgroundColor=" << pp.m_backgroundColor << ",";
 	if (pp.m_listLevelIndex >= 1)
 		o << pp.m_listLevel << ":" << pp.m_listLevelIndex <<",";
 
@@ -205,31 +216,23 @@ void WPSParagraph::addTo(librevenge::RVNGPropertyList &propList, bool inTable) c
 		propList.insert("fo:margin-left", m_listLevelIndex >= 1 ? m_listLevel.m_labelIndent : m_margins[1]);
 		propList.insert("fo:text-indent", m_margins[0]);
 		propList.insert("fo:margin-right", m_margins[2]);
-		if (m_backgroundColor !=  0xFFFFFF)
-		{
-			std::stringstream stream;
-			stream << "#" << std::hex << std::setfill('0') << std::setw(6)
-			       << (m_backgroundColor&0xFFFFFF);
-			propList.insert("fo:background-color", stream.str().c_str());
-		}
+		if (!m_backgroundColor.isWhite())
+			propList.insert("fo:background-color", m_backgroundColor.str().c_str());
 		if (m_border && m_borderStyle.m_style != WPSBorder::None)
 		{
-			std::string style = m_borderStyle.getPropertyValue();
 			int border = m_border;
 			if (border == 0xF)
-			{
-				propList.insert("fo:border", style.c_str());
-			}
+				m_borderStyle.addTo(propList);
 			else
 			{
 				if (border & WPSBorder::LeftBit)
-					propList.insert("fo:border-left", style.c_str());
+					m_borderStyle.addTo(propList, "left");
 				if (border & WPSBorder::RightBit)
-					propList.insert("fo:border-right", style.c_str());
+					m_borderStyle.addTo(propList, "right");
 				if (border & WPSBorder::TopBit)
-					propList.insert("fo:border-top", style.c_str());
+					m_borderStyle.addTo(propList, "top");
 				if (border & WPSBorder::BottomBit)
-					propList.insert("fo:border-bottom", style.c_str());
+					m_borderStyle.addTo(propList, "bottom");
 			}
 		}
 	}
@@ -237,7 +240,41 @@ void WPSParagraph::addTo(librevenge::RVNGPropertyList &propList, bool inTable) c
 	// as we can not use percent, this may give a good approximation
 	propList.insert("fo:margin-top", (10.*m_spacings[1])/72., librevenge::RVNG_INCH);
 	propList.insert("fo:margin-bottom", (10.*m_spacings[2])/72., librevenge::RVNG_INCH);
-	propList.insert("fo:line-height", m_spacings[0] <= 0 ? 1.0 : m_spacings[0], librevenge::RVNG_PERCENT);
+	switch (m_spacingsInterlineType)
+	{
+	case Fixed:
+		if (m_spacings[0] >= 0)
+			propList.insert("fo:line-height", m_spacings[0], m_spacingsInterlineUnit);
+		break;
+	case AtLeast:
+		if (m_spacings[0] <= 0 && m_spacings[0] >= 0)
+			break;
+		if (m_spacings[0] < 0)
+		{
+			static bool first = true;
+			if (first)
+			{
+				WPS_DEBUG_MSG(("WPSParagraph::addTo: interline spacing seems bad\n"));
+				first = false;
+			}
+		}
+		else if (m_spacingsInterlineUnit != librevenge::RVNG_PERCENT)
+			propList.insert("style:line-height-at-least", m_spacings[0], m_spacingsInterlineUnit);
+		else
+		{
+			propList.insert("style:line-height-at-least", m_spacings[0]*12.0, librevenge::RVNG_POINT);
+			static bool first = true;
+			if (first)
+			{
+				first = false;
+				WPS_DEBUG_MSG(("WPSParagraph::addTo: assume height=12 to set line spacing at least with percent type\n"));
+			}
+		}
+		break;
+	default:
+		WPS_DEBUG_MSG(("WPSParagraph::addTo: can not set line spacing type: %d\n",int(m_spacingsInterlineType)));
+		break;
+	}
 	librevenge::RVNGPropertyListVector tabStops;
 	for (size_t i=0; i< m_tabs.size(); i++)
 		m_tabs[i].addTo(tabStops, 0);
